; 3-level call nesting on high-level functions: _LCD_icon, _LCD_fill, _LCD_plot
; (calling a funtion calls another function that calls another function
; can be used on HT48R30+ with an extra function call allowed
;(ints are deferred if no stack)
; 2-level call nesting on low-level functions

include ht48E30.inc

include lcmglob.inc

public _send_data
public _send_cmd
public _send_Adata
public _send_Acmd
public _send_string_RAM
public _send_string_ROM
public _init_LCD
public _LCD_fill
public _LCD_plot
public _LCD_addr
public _LCD_icon
public _sync

public send_data0
public send_cmd0
public send_Adata0
public send_Acmd0
public send_string_RAM0
public send_string_ROM0
public LCD_addr0
public LCD_plot0,LCD_plot1
public LCD_fill0,LCD_fill1,LCD_fill2
public LCD_icon0,LCD_icon1,LCD_icon2


dsec 	.section    'data'
send_string_RAM0 label byte
send_string_ROM0 label byte
send_cmd0	label byte
send_Acmd0	label byte
send_Adata0 label byte
LCD_icon2	label byte
LCD_fill2	label byte
send_data0	db ?
bsychk		db ?
lctr		db ?
LCD_plot0	label byte
LCD_icon0	label byte
LCD_fill0	db ?
LCD_plot1	label byte
LCD_icon1	label byte
LCD_fill1	db ?
LCD_addr0	db 2 dup(?)

scode	.section	'code'					

_send_cmd:
		call busy_chk
		mov a,send_cmd0
		jmp snd_cmd
_send_Acmd:
		call Abusy_chk
		mov a,send_Acmd0
snd_cmd:							; CS=1 (busy check)
snd_:
		mov	lcm_data,a				; bus
		clr	lcm_ctrl.ce				; CE=0
		clr	lcm_ctrl.wr				; WR=0
		set	lcm_ctrl.wr				; WR=1
		set	lcm_ctrl.ce				; CE=1
    	ret                                     	

busy_chk:                                       	
		mov a,3						; STA0, STA1
		mov bsychk,A
		jmp busy
Abusy_chk:                                       	
		mov a,8						; STA3
		mov bsychk,A
busy:	set	lcm_data_ctrl   		; inputs
		set lcm_ctrl.cd  	 		; C/D=1 (cmd)
bsylp:	clr	lcm_ctrl.ce				; CE=0
		clr	lcm_ctrl.rd				; RD=0
		mov A,lcm_data
		set	lcm_ctrl.rd				; RD=1
		set	lcm_ctrl.ce				; CE=1
		and A,bsychk       			; mask out other bits
		xor A,bsychk       			; busy ?
		snz Z
		jmp bsylp	     			; no, loop
		clr lcm_data_ctrl   		; outputs
		ret


_sync:	set	lcm_data_ctrl   		; inputs
		set lcm_ctrl.cd  	 		; C/D=1 (cmd)
		clr	lcm_ctrl.ce				; CE=0
		clr	lcm_ctrl.rd				; RD=0
sylp1:	mov A,lcm_data
		snz ACC.7					; b7=1 ?
		jmp sylp1	     			; no, loop
sylp2:	mov A,lcm_data
		sz ACC.7					; b7=0 ?
		jmp sylp2	     			; no, loop
		set	lcm_ctrl.rd				; RD=1
		set	lcm_ctrl.ce				; CE=1
		clr lcm_data_ctrl   		; outputs
		ret


_send_data:
		call busy_chk
		mov a,send_data0
		jmp snd_dat
_send_Adata:
		call Abusy_chk
		mov a,send_Adata0
snd_dat:
		clr lcm_ctrl.cd  	 		; C/D=0 (dat)
		jmp snd_


_LCD_addr:
		call busy_chk
		mov a,LCD_addr0
		call snd_dat
		call busy_chk
		mov a,LCD_addr0[1]
		call snd_dat
		call busy_chk
		mov a,024h
		jmp snd_cmd

; 0: cols to fill
; 1: rows to fill
; 2: pattern
_LCD_fill:
		call busy_chk
		mov a,0B0h							; auto data 
		call snd_cmd
		mov A,LCD_fill0
		mov lctr,A							; for each col
lfl1:	call Abusy_chk
		mov A,LCD_fill2						; get pattern
		call snd_dat						; put pattern
		sdz lctr							; repeat
		jmp lfl1
		call Abusy_chk
		mov a,0B2h							; reset auto data 
		call snd_cmd
		sdz LCD_fill1						; more rows ?
		jmp lfl2
		ret
lfl2:	call busy_chk						; next row
		mov a,010h
		addm A,LCD_addr0
		clr ACC
		adcm A,LCD_addr0[1]
		call _LCD_addr
		jmp _LCD_fill


; Plot image data, 8 pixels per byte, MSB left */
; 0: x
; 1: y
_LCD_plot:
	mov a,LCD_plot0
	and a,07h
	mov lctr,a
	mov a,7
	subm a,lctr					; 7-x%8
	clr C
	rrc LCD_plot0
	clr C
	rrc LCD_plot0
	clr C
	rrc LCD_plot0				; x/8
	clr LCD_addr0[1]
	clr C
	rlca LCD_plot1
	mov LCD_addr0,a
	rlc LCD_addr0[1]
	rlc LCD_addr0
	rlc LCD_addr0[1]
	rlc LCD_addr0
	rlc LCD_addr0[1]
	rlc LCD_addr0
	rlc LCD_addr0[1]			; y*16
	mov a,LCD_plot0
	addm A,LCD_addr0			; + x/8
	mov A,010h
	adcm A,LCD_addr0[1]			; + 1000h
	call _LCD_addr
	call busy_chk
	mov A,lctr
	or a,0F8h					; set bit
	jmp snd_cmd

; 0: cols
; 1: rows
; 2: icon
_LCD_icon:
		mov A,LCD_icon2
		call seticon
		mov TBLP,A
lil5:	call busy_chk
		mov a,0B0h							; auto data 
		call snd_cmd
		mov A,LCD_icon0
		mov lctr,A							; for each col
lil1:	call Abusy_chk
		mov A,LCD_icon2
		call geticon						; get in A
		call snd_dat
		sdz lctr							; more cols ?
		jmp lil1							; yes
lil4:	call Abusy_chk
		mov a,0B2h							; reset auto data 
		call snd_cmd
		sdz LCD_icon1						; more rows ?
		jmp lil2
		ret
lil2:	call busy_chk						; next row
		mov a,010h
		addm A,LCD_addr0
		clr ACC
		adcm A,LCD_addr0[1]
		call _LCD_addr
		jmp lil5

_send_string_RAM:
		mov A,send_string_RAM0
		mov MP0,A
		call busy_chk
		mov a,0B0h							; auto data 
		call snd_cmd
ssloop:	call Abusy_chk
		mov A,IAR0
		sz ACC
		jmp ssl1
		call Abusy_chk
		mov a,0B2h							; reset auto data 
		jmp snd_cmd
ssl1:	sub A,020h							; ASCII to T6963
		call snd_dat
		inc MP0
		jmp ssloop


ilcd	 	.section INPAGE 'CODE'	
_init_LCD:
		mov a,OFFSET initlcd
		mov TBLP,A
		mov a,4
		mov lctr,A
ill1:	tabrdc ACC
		call snd_dat
		call busy_chk
		mov a,TBLH
		call snd_dat
		call busy_chk
		inc TBLP
		tabrdc ACC
		call snd_cmd
		call busy_chk
		inc TBLP
		sdz lctr
		jmp ill1
		mov a,081h
		call snd_cmd
		call busy_chk				
		mov a,09Ch
		jmp snd_cmd  				

initlcd: dw 00000h,040h, 00010h,041h, 01000h,042h, 00010h,043h	
; TXT home =0, TXT cols = 128, GFX home = 1000h, GFX cols=128


ssrom 	.section INPAGE 'CODE'	
_send_string_ROM:
		call busy_chk
		mov A,0B0h								; auto data
		call snd_cmd
		mov A,send_string_ROM0
		call setstring
		mov TBLP,A
ss2loop:
		call Abusy_chk
		tabrdc ACC		 			
		sz ACC									; end of string ?
		jmp ss2l1								; no, print
		mov A,0B2h								; auto data
		jmp snd_cmd
ss2l1:	sub A,020h								; ASCII to T6963
		call snd_dat
		inc TBLP								
		jmp ss2loop

; strings
txt1:	dw	67,105,107,97,32,69,108,101,99,116,114,111,110,105,99,97,0
txt2:	dw	72,111,108,116,101,107,32,76,67,68,32,100,101,109,111,0

ss	 	.section INPAGE 'CODE'	
setstring:
		addm a,PCL
		ret a,OFFSET txt1
		ret a,OFFSET txt2
		;

si	 	.section INPAGE 'CODE'	
seticon:
		addm a,PCL
		ret a,OFFSET icon0d
		ret a,OFFSET icon1d
		;

gi	 	.section INPAGE 'CODE'	
geticon:
		addm a,PCL
		jmp icon0
		jmp icon1

; data is left to right, up to down
icon0s 	.section INPAGE 'CODE'	
icon0:	tabrdc ACC
		inc TBLP
		ret
icon0d:	; 48x33, 6 cols, 33 rows
        DW      000h,000h,000h,008h,000h,000h,000h,000h,000h,01Ch,000h,000h
        DW      000h,000h,000h,03Ch,000h,020h,000h,000h,000h,07Ch,000h,060h
        DW      000h,000h,000h,0FCh,000h,0E0h,000h,000h,001h,0FCh,001h,0E0h
        DW      000h,000h,003h,0FCh,003h,0E0h,000h,000h,007h,0FCh,007h,0E0h
        DW      000h,000h,00Fh,0FCh,00Fh,0E0h,000h,000h,03Fh,0FCh,01Fh,0E0h
        DW      000h,007h,0FFh,0FFh,0FFh,0E0h,000h,0FFh,0FFh,0F7h,0FFh,0E0h
        DW      003h,081h,0FFh,0E0h,0FFh,0F0h,00Ch,003h,0FFh,0C1h,0FFh,0F8h
        DW      030h,007h,0FFh,083h,0FFh,086h,000h,00Fh,0FFh,007h,0FFh,001h
        DW      040h,01Fh,0FEh,00Fh,0FEh,001h,020h,03Fh,0F8h,03Fh,0FCh,001h
        DW      010h,07Fh,0F0h,07Fh,0F8h,002h,00Eh,0FFh,0E0h,0FFh,0F0h,00Ch
        DW      003h,0FFh,0C1h,0FFh,0E0h,0F0h,003h,0FFh,0E3h,0FFh,0FFh,0C0h
        DW      003h,0FFh,0FFh,0FFh,0FCh,000h,003h,0FEh,00Fh,0FFh,000h,000h
        DW      003h,0FCh,00Fh,0FEh,000h,000h,003h,0F8h,00Fh,0F8h,000h,000h
        DW      003h,0F0h,00Fh,0F0h,000h,000h,003h,0E0h,00Fh,0E0h,000h,000h
        DW      003h,0C0h,00Fh,0C0h,000h,000h,003h,080h,00Fh,080h,000h,000h
        DW      003h,000h,00Fh,000h,000h,000h,000h,000h,00Eh,000h,000h,000h
        DW      000h,000h,00Ch,000h,000h,000h

icon1s 	.section INPAGE 'CODE'
icon1:	tabrdc ACC
		inc TBLP
		ret
icon1d:	; 64x25, 8 cols, 25 rows
        DW      001h,0FEh,078h,001h,0FCh,000h,000h,000h,001h,0F4h,07Eh,003h
        DW      0FCh,000h,000h,000h,001h,0F4h,0FFh,003h,0F8h,000h,000h,000h
        DW      003h,0FCh,0FFh,007h,0F8h,000h,000h,000h,003h,0E8h,0FFh,007h
        DW      0F8h,000h,000h,000h,003h,0E9h,0FFh,007h,0F0h,000h,000h,000h
        DW      007h,0F9h,0FEh,007h,0F0h,000h,000h,000h,007h,0F9h,0F6h,00Fh
        DW      0F0h,000h,000h,000h,007h,0F1h,0ECh,00Eh,073h,0F8h,0FEh,07Eh
        DW      00Fh,0F3h,0D8h,00Ch,063h,0F9h,0FEh,07Fh,00Fh,0F3h,0B0h,01Ch
        DW      067h,0F9h,0FEh,07Fh,00Fh,0E3h,042h,018h,067h,0F1h,0FCh,07Fh
        DW      00Fh,0E6h,00Dh,018h,0C7h,0F3h,0FCh,0FFh,01Fh,0E4h,01Dh,018h
        DW      0CDh,0F3h,0FCh,0FEh,01Fh,0E0h,07Bh,030h,0CDh,0F3h,0F8h,0FEh
        DW      01Fh,0C0h,0FAh,030h,08Dh,0E3h,0F9h,0FEh,03Fh,0C0h,0FEh,031h
        DW      09Bh,0E7h,0F9h,0FEh,03Fh,0C1h,0FEh,071h,09Bh,0E5h,0F1h,07Ch
        DW      03Fh,083h,0FEh,073h,09Bh,0C5h,0F3h,0FCh,03Fh,083h,0FCh,073h
        DW      013h,0CDh,0F3h,0FCh,07Fh,083h,0FCh,077h,033h,0CFh,0F2h,0F8h
        DW      07Fh,003h,0FCh,0FFh,033h,0CBh,0E2h,0F8h,07Fh,001h,0F8h,0FEh
        DW      023h,08Bh,0E6h,0F8h,0FFh,000h,0F8h,0FEh,063h,09Bh,0E5h,0F0h
        DW      000h,000h,000h,0FEh,063h,093h,0C5h,0F0h
		
